#!/usr/bin/env python

from Tkinter import *
import passdb

class mainWindow:
	def __init__(self, master):
		self.master = master
		self.dbObj = passdb.PassDB('mysecretdata.db','1234')
		self._group = ''

		# The box showing groups
		leftscrollbar = Scrollbar(master, orient=VERTICAL)
		self.leftbox = Listbox(master, selectmode=EXTENDED, yscrollcommand=leftscrollbar.set)
		leftscrollbar.config(command=self.leftbox.yview)
		leftscrollbar.grid(row=0, column=9, rowspan=50, sticky=N+S)
		self.leftbox.bind("<Double-Button-1>", self._selectGroup)
		self.leftbox.grid(row=0, rowspan=50, column=0, columnspan=9)
		for item in self.dbObj.groups():
			self.leftbox.insert(END, item)

		# The box showing entries
		rightscrollbar = Scrollbar(master, orient=VERTICAL)
		self.rightbox = Listbox(master, selectmode=EXTENDED, yscrollcommand=rightscrollbar.set)
		self.rightbox.grid(row=0, rowspan=50, column=10, columnspan=9)
		rightscrollbar.config(command=self.rightbox.yview)
		#self.rightbox.bind("<Double-Button-1>", self._selectEntry)
		rightscrollbar.grid(row=0, rowspan=50, column=19, sticky=N+S)

		# The left hand window buttons
		leftaddbutton = Button(master, text="+", command=self._addGroup)
		leftaddbutton.grid(row=50, column=0)
		leftdeletebutton = Button(master, text="-", command=self._delGroup)
		leftdeletebutton.grid(row=50, column=1)
		leftselectbutton = Button(master, text="Select", command=self._selectGroup)
		leftselectbutton.grid(row=50, column=2)

		# The right hand window buttons
		rightaddbutton = Button(master, text="+", command=self._addEntry)
		rightaddbutton.grid(row=50, column=10)
		rightdeletebutton = Button(master, text="-", command=self._delEntry)
		rightdeletebutton.grid(row=50, column=11)
		rightselectbutton = Button(master, text="Select", command=self._selectEntry)
		rightselectbutton.grid(row=50, column=12)

		# The box showing passwords
		self.password = StringVar()
		Entry(master, textvariable=self.password, width=50, relief=SUNKEN).grid(row=100, columnspan=50, column=0)
		
		# The entry action buttons on the right hand side.
		copybutton = Button(master, text="Copy", command=self._copyClipboard)
		copybutton.grid(row=110, column=0)
		clearbutton = Button(master, text="Clear", command=self._clearEntry)
		clearbutton.grid(row=110, column=1)
		showbutton = Button(master, text="Save", command=self._save)
		showbutton.grid(row=110, column=10)
		quitbutton = Button(master, text="Quit", command=self._quit)
		quitbutton.grid(row=110, column=11)

	def _save(self):
		self.dbObj.write()

	def _quit(self):
		self.dbObj.write()
		self.master.quit()

	def _addGroup(self):
		def do_ok():
			newgroup=input.get()
			self.dbObj.add_group(newgroup)
			self.leftbox.delete(0, END)
			for item in self.dbObj.groups():
				self.leftbox.insert(END, item)
			win.destroy()
		def do_cancel():
			win.destroy()
		win = Toplevel() 
		Label(win, text="Group:").grid(row=0, column=0)
		input = Entry(win, width=50, relief=SUNKEN, borderwidth=1)
		input.grid(row=0, column=1, columnspan=2)
		Button(win, text='OK', command=do_ok).grid(row=1, column=1)
		Button(win, text='Cancel', command=do_cancel).grid(row=1, column=2)

	def _delGroup(self):
		groupIdList = self.leftbox.curselection()
		groupList = []
		for GroupId in groupIdList:
			groupList.append(self.leftbox.get(GroupId))
		def do_ok():
			for group in groupList:
				self.dbObj.remove_group(group)
			self.leftbox.delete(0, END)
			for item in self.dbObj.groups():
				self.leftbox.insert(END, item)
			win.destroy()
		def do_cancel():
			win.destroy()
		win = Toplevel() 
		if len(groupList) > 1:
			Label(win, text='Really delete these groups?').grid(row=0, column=0, columnspan=2)
		else:
			Label(win, text='Really delete group "'+groupList[0]+'"?').grid(row=0, column=0, columnspan=2)
		Button(win, text='OK', command=do_ok).grid(row=1, column=0)
		Button(win, text='Cancel', command=do_cancel).grid(row=1, column=1)

	def _addEntry(self):
		if self._group == "":
			return False
		def do_ok():
			newentry=nameInput.get()
			newvalue=valueInput.get()
			self.dbObj.add_entry(self._group,newentry,newvalue)
			self.rightbox.delete(0, END)
			for item in self.dbObj.entries(self._group):
				self.rightbox.insert(END, item)
			win.destroy()
		def do_cancel():
			win.destroy()
		win = Toplevel() 
		Label(win, text="Entry:").grid(row=0, column=0)
		Label(win, text="Password:").grid(row=1, column=0)
		nameInput = Entry(win, width=50, relief=SUNKEN, borderwidth=1)
		nameInput.grid(row=0, column=1, columnspan=2)
		valueInput = Entry(win, show='*', width=50, relief=SUNKEN, borderwidth=1)
		valueInput.grid(row=1, column=1, columnspan=2)
		Button(win, text='OK', command=do_ok).grid(row=2, column=1)
		Button(win, text='Cancel', command=do_cancel).grid(row=2, column=2)

	def _delEntry(self):
		if self._group == "":
			return False
		entryIdList = self.rightbox.curselection()
		entryList = []
		for entryId in entryIdList:
			entryList.append(self.rightbox.get(entryId))
		def do_ok():
			for entry in entryList:
				self.dbObj.remove_entry(self._group,entry)
			self.rightbox.delete(0, END)
			for item in self.dbObj.entries(self._group):
				self.rightbox.insert(END, item)
			win.destroy()
		def do_cancel():
			win.destroy()
		win = Toplevel() 
		if len(entryList) > 1:
			Label(win, text='Really delete these entries?').grid(row=0, column=0, columnspan=2)
		else:
			Label(win, text='Really delete entry "'+entryList[0]+'"?').grid(row=0, column=0, columnspan=2)
		Button(win, text='OK', command=do_ok).grid(row=1, column=0)
		Button(win, text='Cancel', command=do_cancel).grid(row=1, column=1)

	def _selectGroup(self, null=None):
		self._group = self.leftbox.get(ACTIVE)
		self.rightbox.delete(0, END)
		for item in self.dbObj.entries(self._group):
			self.rightbox.insert(END, item)
		
	def _selectEntry(self, null=None):
		entry = self.rightbox.get(ACTIVE)
		self.password.set(self.dbObj.get(self._group,entry))

	def _copyClipboard(self):
		entry = self.rightbox.get(ACTIVE)
		root.clipboard_clear()
		root.clipboard_append(self.dbObj.get(self._group,entry))
		
	def _clearEntry(self):
		self.password.set('')


root = Tk()
app = mainWindow(root)
root.mainloop()
